# WARNING: Please don't edit this file. It was generated by Python/WinRT v3.0.0

import enum

import winrt.system
from winrt._winrt_windows_services_targetedcontent import (
    TargetedContentAction,
    TargetedContentAvailabilityChangedEventArgs,
    TargetedContentChangedEventArgs,
    TargetedContentCollection,
    TargetedContentContainer,
    TargetedContentFile,
    TargetedContentImage,
    TargetedContentItem,
    TargetedContentItemState,
    TargetedContentObject,
    TargetedContentStateChangedEventArgs,
    TargetedContentSubscription,
    TargetedContentSubscriptionOptions,
    TargetedContentValue,
)

__all__ = [
    "TargetedContentAppInstallationState",
    "TargetedContentAvailability",
    "TargetedContentInteraction",
    "TargetedContentObjectKind",
    "TargetedContentValueKind",
    "TargetedContentAction",
    "TargetedContentAvailabilityChangedEventArgs",
    "TargetedContentChangedEventArgs",
    "TargetedContentCollection",
    "TargetedContentContainer",
    "TargetedContentFile",
    "TargetedContentImage",
    "TargetedContentItem",
    "TargetedContentItemState",
    "TargetedContentObject",
    "TargetedContentStateChangedEventArgs",
    "TargetedContentSubscription",
    "TargetedContentSubscriptionOptions",
    "TargetedContentValue",
]

class TargetedContentAppInstallationState(enum.IntEnum):
    NOT_APPLICABLE = 0
    NOT_INSTALLED = 1
    INSTALLED = 2

class TargetedContentAvailability(enum.IntEnum):
    NONE = 0
    PARTIAL = 1
    ALL = 2

class TargetedContentInteraction(enum.IntEnum):
    IMPRESSION = 0
    CLICK_THROUGH = 1
    HOVER = 2
    LIKE = 3
    DISLIKE = 4
    DISMISS = 5
    INELIGIBLE = 6
    ACCEPT = 7
    DECLINE = 8
    DEFER = 9
    CANCELED = 10
    CONVERSION = 11
    OPPORTUNITY = 12

class TargetedContentObjectKind(enum.IntEnum):
    COLLECTION = 0
    ITEM = 1
    VALUE = 2

class TargetedContentValueKind(enum.IntEnum):
    STRING = 0
    URI = 1
    NUMBER = 2
    BOOLEAN = 3
    FILE = 4
    IMAGE_FILE = 5
    ACTION = 6
    STRINGS = 7
    URIS = 8
    NUMBERS = 9
    BOOLEANS = 10
    FILES = 11
    IMAGE_FILES = 12
    ACTIONS = 13

